--[[

DarkModeDisplay

Automatically switch Displays to DarkMode on sunset

Author:		Bone2510 @LSFM - lsfarming-mods.com
Date:		17.07.2023

Version:	v1.3

	   
History:	v1.0 @21.07.2022 - initial implementation in FS 22 - possibility to Switch between auto and manual mode
			--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
			v1.1 @07.03.2023 - added multiplayer synchronization for auto and manual mode
			--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
            v1.2 @05.07.2023 - Reworked script to take as many displays as you want.
            --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
            v1.3 @05.07.2023 - fixed MP and other issues
            --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
]]

--@class DarkModeDisplay
DarkModeDisplay = {};
DarkModeDisplay.MOD_NAME = g_currentModName;
DarkModeDisplay.BASE_KEY = "vehicle.darkModeDisplays.display";
--DarkModeDisplay.DISPLAYS = {};


function DarkModeDisplay.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Dashboard, specializations);
end

function DarkModeDisplay.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onPreLoad", DarkModeDisplay);
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", DarkModeDisplay);
    SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", DarkModeDisplay);
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", DarkModeDisplay);
    SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", DarkModeDisplay);
    SpecializationUtil.registerEventListener(vehicleType, "onReadStream", DarkModeDisplay);
    SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", DarkModeDisplay);
end

function DarkModeDisplay.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "enableDarkMode", DarkModeDisplay.enableDarkMode);
    SpecializationUtil.registerFunction(vehicleType, "disableDarkMode", DarkModeDisplay.disableDarkMode);
    SpecializationUtil.registerFunction(vehicleType, "modeHandler", DarkModeDisplay.modeHandler);
end

function DarkModeDisplay.registerEvents(vehicleType)
    SpecializationUtil.registerEvent(vehicleType, "AutoDarkModeEvent")
    SpecializationUtil.registerEvent(vehicleType, "ManualDarkModeEvent")
end

function DarkModeDisplay.initSpecialization(specializations)
    local schema = Vehicle.xmlSchema;
    schema:setXMLSpecializationType("darkModeDisplay");

    schema:register(XMLValueType.STRING, DarkModeDisplay.BASE_KEY .. "#node", "Get display", false);
    schema:register(XMLValueType.STRING, DarkModeDisplay.BASE_KEY .. "#replaceNode", "Get dark display", false);
    schema:register(XMLValueType.STRING, DarkModeDisplay.BASE_KEY .. "#dashboardNode", "Get dashboard node", false);
    schema:register(XMLValueType.STRING, DarkModeDisplay.BASE_KEY .. "#replaceDashboardNode", "Get dark dashboard node", false);

    schema:setXMLSpecializationType();
end

function DarkModeDisplay:onPreLoad(savegame)
    self.spec_darkModeDisplay = self[string.format("spec_%s.darkModeDisplay", DarkModeDisplay.MOD_NAME)];
    local spec = self.spec_darkModeDisplay;

    spec.autoDarkMode = true;
    spec.darkModeActive = false;
    spec.DISPLAYS = {};
    spec.isInitialized = false;
end

function DarkModeDisplay:onLoad(savegame)

    print("Load called!");
    --DarkModeDisplay.DISPLAYS = {};
    local spec = self.spec_darkModeDisplay;

    local displayNumber = 0;
    local displayKey = DarkModeDisplay.BASE_KEY .. "(" ..tostring(displayNumber) .. ")";

    while self.xmlFile:hasProperty(displayKey) do

        displayKey = DarkModeDisplay.BASE_KEY .. "(" ..tostring(displayNumber) .. ")";

        local display = {};
        local displayNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile.handle ,displayKey .. "#node"), self.i3dMappings);

        if displayNode ~= nil then
            display.displayNode = displayNode;
        end


        local replaceDisplayNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile.handle ,displayKey .. "#replaceNode"), self.i3dMappings);

        if replaceDisplayNode ~= nil then
            display.replaceDisplayNode = replaceDisplayNode;
        end


        local dashboardNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile.handle ,displayKey .. "#dashboardNode"), self.i3dMappings);

        if dashboardNode ~= nil then
            display.dashboardNode = dashboardNode;
        end


        local replaceDashboardNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile.handle, displayKey .. "#replaceDashboardNode"), self.i3dMappings);

        if replaceDashboardNode ~= nil then
            display.replaceDashboardNode = replaceDashboardNode;
        end

        table.insert(spec.DISPLAYS, display);
        displayNumber = displayNumber + 1;

    end
    spec.isInitialized = true;
end

function DarkModeDisplay:onPostLoad(savegame)
    local spec = self.spec_darkModeDisplay;
    if savegame ~= nil then
        local xmlFile = savegame.xmlFile;
        local iterationKey = savegame.key .. "." .. DarkModeDisplay.MOD_NAME .. ".darkModeDisplay";
        
        spec.autoDarkMode = true;
        spec.darkModeActive = false;

        if hasXMLProperty(xmlFile.handle, iterationKey .. "#autoDarkMode") and hasXMLProperty(xmlFile.handle, iterationKey .. "#darkModeActive") then
            spec.autoDarkMode = getXMLBool(xmlFile.handle, iterationKey .. "#autoDarkMode");
            spec.darkModeActive = getXMLBool(xmlFile.handle, iterationKey .. "#darkModeActive");
        end
    end
end

function DarkModeDisplay:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
    if self.isClient then
        local spec = self.spec_darkModeDisplay;
        self:clearActionEventsTable(spec.actionEvents)

        if isActiveForInputIgnoreSelection then
            local _, actionEventId = self:addActionEvent(spec.actionEvents, "DD_toggleAutoSwitch", self, DarkModeDisplay.actionEventDD_toggleAutoSwitch, false, true, false, true, nil);
            g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL);
            --g_inputBinding:setActionEventText(actionEventId, g_i18n:getText("input_DD_toggleAutoSwitchDisable"));
            spec.actionEventId = actionEventId;

            local _, actionEventId1 = self:addActionEvent(spec.actionEvents, "DD_toggleManualDark", self, DarkModeDisplay.actionEventDD_toggleManualDark, false, true, false, true, nil);
            g_inputBinding:setActionEventTextPriority(actionEventId1, GS_PRIO_NORMAL);
            --g_inputBinding:setActionEventText(actionEventId1, g_i18n:getText("input_DD_toggleManualDark"));
            spec.actionEventId1 = actionEventId1;
            DarkModeDisplay.updateActionEvents(self);
        end
    end
end

function DarkModeDisplay:updateActionEvents()
    local spec = self.spec_darkModeDisplay;

    g_inputBinding:setActionEventText(spec.actionEventId, spec.autoDarkMode and g_i18n:getText("input_DD_toggleAutoSwitchDisable") or g_i18n:getText("input_DD_toggleAutoSwitchEnable"));
    g_inputBinding:setActionEventText(spec.actionEventId1, spec.darkModeActive and g_i18n:getText("input_DD_toggleManualDarkDisable") or g_i18n:getText("input_DD_toggleManualDarkEnable"));

    g_inputBinding:setActionEventActive(spec.actionEventId1, not spec.autoDarkMode);
end

function DarkModeDisplay:saveToXMLFile(xmlFile, key, usedModNames)
    local spec = self.spec_darkModeDisplay;

    setXMLBool(xmlFile.handle, key .. "#autoDarkMode", spec.autoDarkMode);
    setXMLBool(xmlFile.handle, key .. "#darkModeActive", spec.darkModeActive)
end

---Called on update
function DarkModeDisplay:onUpdate(dt)
    local spec = self.spec_darkModeDisplay;
    if spec.isInitialized then
        if self.getIsMotorStarted ~= nil then
            if self:getIsMotorStarted() then
                DarkModeDisplay:modeHandler(spec);
            else
                DarkModeDisplay:disable(spec);
            end
        end
    end
end

function DarkModeDisplay:modeHandler(spec) 
    local autoDarkMode = spec.autoDarkMode;
    local needDarkMode = not g_currentMission.environment.isSunOn;
    if needDarkMode and autoDarkMode then 
        spec.darkModeActive = true;
        DarkModeDisplay:enableDarkMode(spec);
    end

    if not needDarkMode and autoDarkMode then
        spec.darkModeActive = false;
        DarkModeDisplay:disableDarkMode(spec);
    end

    if not autoDarkMode then
        if spec.darkModeActive then
            DarkModeDisplay:enableDarkMode(spec);
        end
        if not spec.darkModeActive then
            DarkModeDisplay:disableDarkMode(spec);
        end
    end

end

function DarkModeDisplay:disable(spec)
    for i, display in pairs(spec.DISPLAYS) do
        if display.dashboardNode ~= nil and display.replaceDashboardNode ~= nil then
            setVisibility(display.dashboardNode, false);
            setVisibility(display.replaceDashboardNode, false);
        end
    end
end

function DarkModeDisplay.actionEventDD_toggleAutoSwitch(self, actionName, inputValue, callbackState, isAnalog)
    local spec = self.spec_darkModeDisplay;
    local multiplayer = g_currentMission.missionDynamicInfo.isMultiplayer

    spec.autoDarkMode = not spec.autoDarkMode;
    DarkModeDisplay.updateActionEvents(self);
    AutoDarkModeEvent.sendEvent(self, spec.autoDarkMode, false)
end

function DarkModeDisplay.actionEventDD_toggleManualDark(self, actionName, inputValue, callbackState, isAnalog)
    local spec = self.spec_darkModeDisplay;

    if spec.autoDarkMode then
        return
    end

    spec.darkModeActive = not spec.darkModeActive;
    DarkModeDisplay.updateActionEvents(self);
    ManualDarkModeEvent.sendEvent(self, spec.darkModeActive, false)
end

function DarkModeDisplay:enableDarkMode(spec)
    for i, display in pairs(spec.DISPLAYS) do

        if display.displayNode ~= nil and display.replaceDisplayNode ~= nil then
            setIsNonRenderable(display.displayNode, true);
            setIsNonRenderable(display.replaceDisplayNode, false);
        end

        if display.dashboardNode ~= nil and display.replaceDashboardNode ~= nil then
            setVisibility(display.dashboardNode, false);
            setVisibility(display.replaceDashboardNode, true);
        end
    end
end

function DarkModeDisplay:disableDarkMode(spec)
    for i, display in ipairs(spec.DISPLAYS) do
        if (display.displayNode ~= nil and display.replaceDisplayNode ~= nil ) then
            setIsNonRenderable(display.displayNode, false);
            setIsNonRenderable(display.replaceDisplayNode, true);
        end

        if(display.dashboardNode ~= nil and display.replaceDashboardNode ~= nil) then
            setVisibility(display.dashboardNode, true);
            setVisibility(display.replaceDashboardNode, false);
        end
    end
end

-- Multiplayer Sync Part

function DarkModeDisplay:onWriteStream(streamId, connection)
    local spec = self.spec_darkModeDisplay
    streamWriteBool(streamId, spec.autoDarkMode)
    streamWriteBool(streamId, spec.darkModeActive)
end

function DarkModeDisplay:onReadStream(streamId, connection)
    local autoDarkMode = streamReadBool(streamId)
    local darkModeActive = streamReadBool(streamId)

    self.spec_darkModeDisplay.autoDarkMode = autoDarkMode
    self.spec_darkModeDisplay.darkModeActive = darkModeActive

    local spec = self.spec_darkModeDisplay;
    DarkModeDisplay:modeHandler(spec);

end

-- Auto Mode Sync

AutoDarkModeEvent = {}

local AutoDarkModeEvent_mt = Class(AutoDarkModeEvent, Event)

InitEventClass(AutoDarkModeEvent, "AutoDarkModeEvent")

function AutoDarkModeEvent:emptyNew()
    local self = Event.new(AutoDarkModeEvent_mt)
    return self
end

function AutoDarkModeEvent.new(vehicle, active)
    local self = AutoDarkModeEvent:emptyNew()
    self.vehicle = vehicle
    self.active = active

    return self
end

function AutoDarkModeEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObject(streamId, self.vehicle)
    streamWriteBool(streamId, self.active)
end

function AutoDarkModeEvent:readStream(streamId, connection)
    self.vehicle = NetworkUtil.readNodeObject(streamId)
    self.active = streamReadBool(streamId)
    self:run(connection)
end

function AutoDarkModeEvent:run(connection)
    if not connection:getIsServer() then
        g_server:broadcastEvent(self, false, connection, self.vehicle)
    end
    if self.vehicle ~= nil then
        self.vehicle.spec_darkModeDisplay.autoDarkMode = self.active
        DarkModeDisplay.updateActionEvents(self.vehicle);
    end
end

function AutoDarkModeEvent.sendEvent(vehicle, active, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(AutoDarkModeEvent.new(vehicle, active), nil, nil, vehicle)
        else
            g_client:getServerConnection():sendEvent(AutoDarkModeEvent.new(vehicle, active))
        end
    end
end

-- Manual Mode Sync

ManualDarkModeEvent = {}

local ManualDarkModeEvent_mt = Class(ManualDarkModeEvent, Event)

InitEventClass(ManualDarkModeEvent, "ManualDarkModeEvent")

function ManualDarkModeEvent:emptyNew()
    local self = Event.new(ManualDarkModeEvent_mt)
    return self
end

function ManualDarkModeEvent.new(vehicle, active)
    local self = ManualDarkModeEvent:emptyNew()
    self.vehicle = vehicle
    self.active = active

    return self
end

function ManualDarkModeEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObject(streamId, self.vehicle)
    streamWriteBool(streamId, self.active)
end

function ManualDarkModeEvent:readStream(streamId, connection)
    self.vehicle = NetworkUtil.readNodeObject(streamId)
    self.active = streamReadBool(streamId)
    self:run(connection)
end

function ManualDarkModeEvent:run(connection)
    if not connection:getIsServer() then
        g_server:broadcastEvent(self, false, connection, self.vehicle)
    end
    if self.vehicle ~= nil then
        self.vehicle.spec_darkModeDisplay.darkModeActive = self.active
        DarkModeDisplay.updateActionEvents(self.vehicle);
    end
end

function ManualDarkModeEvent.sendEvent(vehicle, active, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(ManualDarkModeEvent.new(vehicle, active), nil, nil, vehicle)
        else
            g_client:getServerConnection():sendEvent(ManualDarkModeEvent.new(vehicle, active))
        end
    end
end