--[[
	BaseGameFixes
	Specialization to fix some issues from the base game!
	
	@author:    Ifko[nator]
	@date:      10.09.2022
	@version:	2.4

	History:	v1.0 @21.11.2020 - initial implementation in FS 19
				-------------------------------------------------------------------------------------------------------------------
				v1.1 @04.01.2021 - added fix for consoleCommandSetFOV
				-------------------------------------------------------------------------------------------------------------------
				v1.2 @16.01.2021 - shopFoldingState Fix. @GIANTS: You must use getXMLInt instead of getXMLBool! N00BS! --## removed
				-------------------------------------------------------------------------------------------------------------------
				v1.3 @16.02.2021 - added fix for FillLevelsDisplay.lua --## removed
				-------------------------------------------------------------------------------------------------------------------
				v1.4 @22.03.2021 - fixed component joint rot limit for new frontloader joint types
				-------------------------------------------------------------------------------------------------------------------
				v2.0 @26.12.2021 - convert to FS 22
				-------------------------------------------------------------------------------------------------------------------
				v2.1 @11.03.2022 - added foldable l10n text fix for mods
				-------------------------------------------------------------------------------------------------------------------
				v2.2 @23.03.2022 - added communal orange Fix
				-------------------------------------------------------------------------------------------------------------------
				v2.3 @15.08.2022 - added getAccelerationAxis and getDecelerationAxis Fix
				-------------------------------------------------------------------------------------------------------------------
				v2.4 @10.09.2022 - fixed "objectText" value for foldable
]]

BaseGameFixes = {};
BaseGameFixes.currentModName = g_currentModName;

---------------------------------------------------------------------------------------------------------------------------------------------------
--## MotorizedFix
function BaseGameFixes:onWriteUpdateStreamMotorized(streamId, connection, dirtyMask)
    if not connection.isServer then
		if JohnDeerePackUtil == nil then
			return;
		end;
	
		local specMotorized = JohnDeerePackUtil.getSpecByName(self, "motorized");

        if streamWriteBool(streamId, specMotorized.isMotorStarted) then
			streamWriteInt8(streamId, specMotorized.motorTemperature.value, 7);
			streamWriteInt8(streamId, specMotorized.lastFuelUsage, 7);
        end;
    end;
end;

function BaseGameFixes:onReadUpdateStreamMotorized(streamId, timestamp, connection)
    if connection.isServer then
        if JohnDeerePackUtil == nil then
			return;
		end;
	
		local specMotorized = JohnDeerePackUtil.getSpecByName(self, "motorized");

        if streamReadBool(streamId) then
			specMotorized.motorTemperature.value = streamReadUInt8(streamId);
			specMotorized.lastFuelUsage = streamReadUInt8(streamId);
        end;
    end;
end;

Motorized.onWriteUpdateStream = Utils.appendedFunction(Motorized.onWriteUpdateStream, BaseGameFixes.onWriteUpdateStreamMotorized);
Motorized.onReadUpdateStream = Utils.appendedFunction(Motorized.onReadUpdateStream, BaseGameFixes.onReadUpdateStreamMotorized);
---------------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------------
--## AttacherJointsFix
function BaseGameFixes:loadAttacherJointFromXML(superFunc, attacherJoint, xmlFile, baseName, index)
	local attacherJoints = superFunc(self, attacherJoint, xmlFile, baseName, index);

	if attacherJoints ~= nil then
		local schemaKey = baseName .. ".schema";

		local loadedXMLFile = loadXMLFile("vehicle", xmlFile.filename);
		
		attacherJoint.invertX = Utils.getNoNil(getXMLBool(loadedXMLFile, schemaKey .. "#invertX"), false);

		delete(loadedXMLFile);
	end;

	return attacherJoints;
end;

AttacherJoints.loadAttacherJointFromXML = Utils.overwrittenFunction(AttacherJoints.loadAttacherJointFromXML, BaseGameFixes.loadAttacherJointFromXML);
---------------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------------
--## Fov fix
function BaseGameFixes:consoleCommandSetFOV(fovY)
	if self.controlledVehicle ~= nil then
        local cam = self.controlledVehicle:getActiveCamera();
		
		if cam.oldFovSaved ~= nil then
			cam.oldFovSaved = math.min(fovY);
		end;
    end;
end;

BaseMission.consoleCommandSetFOV = Utils.prependedFunction(BaseMission.consoleCommandSetFOV, BaseGameFixes.consoleCommandSetFOV);
---------------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------------
--## Frontlader attacher fix
function BaseGameFixes:onPreAttachImplement(superFunc, attachable, inputJointDescIndex, jointDescIndex)
	if JohnDeerePackUtil == nil then
		return superFunc(self, attachable, inputJointDescIndex, jointDescIndex);
	end;

	local specFrontloaderAttacher = JohnDeerePackUtil.getSpecByName(self, "frontloaderAttacher");

	if specFrontloaderAttacher ~= nil and specFrontloaderAttacher.frontAxisJoint ~= nil then
		local attacherJoint = nil;
		local attacherJoints = self:getAttacherJoints();

		if attacherJoints ~= nil then
			attacherJoint = attacherJoints[jointDescIndex];
		end;

		local isFrontloaderAttacher = JohnDeerePackUtil.getIsFrontloaderAttcher(attacherJoint.jointType);

		if attacherJoint ~= nil and isFrontloaderAttacher and self.isServer then
			specFrontloaderAttacher.rotLimit = {
				unpack(self.componentJoints[specFrontloaderAttacher.frontAxisJoint].rotLimit)
			};

			for i = 1, 3 do
				self:setComponentJointRotLimit(self.componentJoints[specFrontloaderAttacher.frontAxisJoint], i, 0, 0);
			end;
		end;
	end;
end;

FrontloaderAttacher.onPreAttachImplement = Utils.overwrittenFunction(FrontloaderAttacher.onPreAttachImplement, BaseGameFixes.onPreAttachImplement);
---------------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------------
--## MaterialUtilFix
function BaseGameFixes.validateMaterialAttributes(id, superFunc, sourceFunc)
	local fillTypeStr = getUserAttribute(id, "fillType");

	if fillTypeStr == nil then
		print("Warning: No fillType given in '" .. getName(id) .. "' for " .. sourceFunc);

		return false;
	end;

	local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillTypeStr);

	if fillTypeIndex == nil then
		--print("Warning: Unknown fillType '" .. tostring(fillTypeStr) .. "' for " .. sourceFunc);

		return false;
	end;

	local materialTypeName = getUserAttribute(id, "materialType");

	if materialTypeName == nil then
		print("Warning: No materialType given for '" .. getName(id) .. "' for " .. sourceFunc);

		return false;
	end;

	local materialType = g_materialManager:getMaterialTypeByName(materialTypeName);

	if materialType == nil then
		print("Warning: Unknown materialType '" .. materialTypeName .. "' given for '" .. getName(id) .. "' for " .. sourceFunc);

		return false;
	end;

	local matIdStr = Utils.getNoNil(getUserAttribute(id, "materialIndex"), 1);
	local materialIndex = tonumber(matIdStr);

	if materialIndex == nil then
		print("Warning: Invalid materialIndex '" .. matIdStr .. "' for " .. getName(id) .. "-" .. materialTypeName .. "!");

		return false;
	end;

	return true, fillTypeIndex, materialType, materialIndex;
end;

MaterialUtil.validateMaterialAttributes = Utils.overwrittenFunction(MaterialUtil.validateMaterialAttributes, BaseGameFixes.validateMaterialAttributes);
---------------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------------
--## Foldable l10n text fix
function BaseGameFixes:onLoadFoldable(savegame)
	local specFoldable = self.spec_foldable
	
	local foldableConfigKey = string.format("vehicle.foldable.foldingConfigurations.foldingConfiguration(%d).foldingParts", Utils.getNoNil(self.configurations.folding, 1) - 1);

	local function getL10nText(text)
		if text:sub(1, 6) == "$l10n_" then
			text = g_i18n:getText(text:sub(7), BaseGameFixes.currentModName);
		elseif g_i18n:hasText(text, BaseGameFixes.currentModName) then
			text = g_i18n:getText(text, BaseGameFixes.currentModName);
		end;
	
		return text;
	end;

	specFoldable.objectText = getL10nText(Utils.getNoNil(getXMLString(self.xmlFile.handle, foldableConfigKey .. "#objectText"), self.typeDesc));
	specFoldable.posDirectionText = string.format(getL10nText(Utils.getNoNil(getXMLString(self.xmlFile.handle, foldableConfigKey .. "#posDirectionText"), "action_foldOBJECT")), self.typeDesc);
	specFoldable.negDirectionText = string.format(getL10nText(Utils.getNoNil(getXMLString(self.xmlFile.handle, foldableConfigKey .. "#negDirectionText"), "action_foldOBJECT")), self.typeDesc);
	specFoldable.middlePosDirectionText = string.format(getL10nText(Utils.getNoNil(getXMLString(self.xmlFile.handle, foldableConfigKey .. "#middlePosDirectionText"), "action_liftOBJECT")), self.typeDesc);
	specFoldable.middleNegDirectionText = string.format(getL10nText(Utils.getNoNil(getXMLString(self.xmlFile.handle, foldableConfigKey .. "#middleNegDirectionText"), "action_lowerOBJECT")), self.typeDesc);
	specFoldable.unfoldWarning = string.format(getL10nText(Utils.getNoNil(getXMLString(self.xmlFile.handle, foldableConfigKey .. "#unfoldWarning"), "warning_firstUnfoldTheTool")), specFoldable.objectText);
	specFoldable.detachWarning = string.format(getL10nText(Utils.getNoNil(getXMLString(self.xmlFile.handle, foldableConfigKey .. "#detachWarning"), "warning_doNotDetachWhileFolding")), specFoldable.objectText);
end;

Foldable.onLoad = Utils.appendedFunction(Foldable.onLoad, BaseGameFixes.onLoadFoldable);
---------------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------------
--## Communal Orange Fix
g_brandColorManager.brandColors["SAMASZ_ORANGE"] = g_brandColorManager.brandColors["FENDT_ORANGE1"];
---------------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------------
--## Drivable getAccelerationAxis fix
function BaseGameFixes:getAccelerationAxisDrivable()
	local dir = self.movingDirection > -1 and 1 or -1;
	
	if self.movingDirection < 0 then
		dir = 1;
	end;

	return math.max(self.spec_drivable.axisForward * dir * self:getReverserDirection(), 0);
end;

Drivable.getAccelerationAxis = Utils.overwrittenFunction(Drivable.getAccelerationAxis, BaseGameFixes.getAccelerationAxisDrivable);
---------------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------------
--## Drivable getDecelerationAxis fix
function BaseGameFixes:getDecelerationAxisDrivable()
	if self.lastSpeedReal > 0.0001 then
		local decelerationAxisInput = math.abs(math.min(self.spec_drivable.axisForward * self.movingDirection * self:getReverserDirection(), 0));
		
		if self.movingDirection < 0 then
			decelerationAxisInput = math.max(self.spec_drivable.axisForward * self.movingDirection * self:getReverserDirection(), 0);
		end;

		return decelerationAxisInput;
	end;

	return 0;
end;

Drivable.getDecelerationAxis = Utils.overwrittenFunction(Drivable.getDecelerationAxis, BaseGameFixes.getDecelerationAxisDrivable);
---------------------------------------------------------------------------------------------------------------------------------------------------

---- Reverse lights fix ---------------------------------------------------------------------------------------------------------------------------

function BaseGameFixes:onVehiclePhysicsUpdate(superFunc, acceleratorPedal, brakePedal, automaticBrake, currentSpeed)
	self:setBrakeLightsVisibility(not automaticBrake and math.abs(brakePedal) > 0)

	local reverserDirection = 1

	if self.spec_drivable ~= nil then
		reverserDirection = self.spec_drivable.reverserDirection
	end

	if self.spec_motorized == nil then
		self:setReverseLightsVisibility((currentSpeed < -self.spec_lights.reverseLightActivationSpeed or acceleratorPedal < 0) and reverserDirection == 1)
	elseif self.spec_motorized ~= nil then
		self:setReverseLightsVisibility(self.spec_motorized.motor:getDrivingDirection() < 0 and self:getIsMotorStarted());
	end
end

Lights.onVehiclePhysicsUpdate = Utils.overwrittenFunction(Lights.onVehiclePhysicsUpdate, BaseGameFixes.onVehiclePhysicsUpdate)

---------------------------------------------------------------------------------------------------------------------------------------------------